//
//  TKTextField.swift
//  TKCocoaModule
//
//  Created by 🐶 on 12/21/21.
//
import Cocoa

public class TKTextField: NSTextField {

    public override var maximumNumberOfLines: Int {
        set {
            super.maximumNumberOfLines = 1
        }
        get {
            return super.maximumNumberOfLines
        }
    }
    
    
    // 指定一个cell
    private let _cell = TextFieldCell.init()
    public override var cell: NSCell? {
        set {
            super.cell = _cell
        }
        get {
            return super.cell
        }
    }
    
    
    public override init(frame frameRect: NSRect) {
        super.init(frame: frameRect)
        self.wantsLayer = true
    }
    
    public override var backgroundColor: NSColor? {
        didSet {
            self.layer?.backgroundColor  = backgroundColor?.cgColor
        }
    }
    
    @available(*, message:"don`t set wantsLayer = false")
    public override var wantsLayer: Bool {
        set {
            super.wantsLayer = true
        }
        get {
            return super.wantsLayer
        }
    }
    
    
    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
}


class TextFieldCell : NSTextFieldCell {

    override var isBezeled: Bool {
        set {

        }
        get {
            return false
        }
    }

    override var bezelStyle: NSTextField.BezelStyle {
        set {

        }
        get {
            return .squareBezel
        }
    }

    override var drawsBackground: Bool {
        set {

        }
        get {
            return false
        }
    }

    override var focusRingType: NSFocusRingType {
        set {

        }
        get {
            return .none
        }
    }

    override var importsGraphics: Bool {
        set {

        }
        get {
            return false
        }
    }

    
    
    override init(textCell string: String) {
        super.init(textCell: string)
        self.bezelStyle = .roundedBezel
        self.isBezeled = false
        self.isBordered = false
        self.focusRingType = .none
        self.usesSingleLineMode  = true
        self.truncatesLastVisibleLine = false
        self.isEditable = true
        self.drawsBackground = false
        self.isScrollable = true
    }
    
    required init(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    func adjustedFrame(toVerticallyCenterText rect: NSRect) -> NSRect {
          var titleRect = self.titleRect(forBounds: rect)
          let cellSize = self.cellSize(forBounds: rect)
          let minimumHeight = cellSize.height
          titleRect.origin.y += (titleRect.height - minimumHeight) / 2
          titleRect.size.height = minimumHeight

          return titleRect
      }
    // TODO: 编辑模式下的
    override func edit(withFrame rect: NSRect, in controlView: NSView, editor textObj: NSText, delegate: Any?, event: NSEvent?) {
        super.edit(withFrame: adjustedFrame(toVerticallyCenterText: rect), in: controlView, editor: textObj, delegate: delegate, event: event)
    }
    // TODO: 选中模式下的
    override func select(withFrame rect: NSRect, in controlView: NSView, editor textObj: NSText, delegate: Any?, start selStart: Int, length selLength: Int) {
        super.select(withFrame: adjustedFrame(toVerticallyCenterText: rect), in: controlView, editor: textObj, delegate: delegate, start: selStart, length: selLength)
    }
    
    // TODO: 普通模式下的
    override func drawInterior(withFrame cellFrame: NSRect, in controlView: NSView) {
        let borderWidth = controlView.layer?.borderWidth ?? 0
        let rect = CGRect.init(x: cellFrame.origin.x,
                               y: cellFrame.origin.y,
                               width: cellFrame.size.width - borderWidth * 2,
                               height: cellFrame.size.height - borderWidth * 2)
        super.drawInterior(withFrame: adjustedFrame(toVerticallyCenterText: rect), in: controlView)
    }
}


extension TextFieldCell {
    
    // TODO: 研究是否可以去掉
    override func drawingRect(forBounds rect: NSRect) -> NSRect {
        let boarderWidth = self.controlView?.layer?.borderWidth ?? 0
        let value = super.drawingRect(forBounds:CGRect.init(x: boarderWidth, y: rect.minY, width: rect.width - boarderWidth, height: rect.height))
        return value
    }
}

// TODO: 处理 isBordered = false， 但是又设置了 layer.boarderWidth 的情况
extension TextFieldCell {
    override func titleRect(forBounds rect: NSRect) -> NSRect {
        let value = super.titleRect(forBounds: rect)
        let boarderWidth = self.controlView?.layer?.borderWidth ?? 0
        if !isBordered {
            return NSRect.init(x: boarderWidth, y: rect.minY, width: value.width - boarderWidth * 2, height: value.height - value.minY * 2)
        }
        return value
    }
    
    override func cellSize(forBounds rect: NSRect) -> NSSize {
        let boarderWidth = self.controlView?.layer?.borderWidth ?? 0
        let value = super.cellSize(forBounds: rect)
        if !isBordered {
            return NSSize.init(width: value.width + boarderWidth / 2, height: value.height + boarderWidth / 2)
        }
        return value
    }
}
